#include "configuration.h"
#include "figure.h"

#include <cassert>  // assert
#ifndef NDEBUG
#include <iostream>  // only for debug output
#endif

namespace nqueens{
   int maximum_figure_ID = -1;
}

using namespace nqueens;

void nqueens::Configuration::add_queen_at(int x, int y)
{
   assert(!this->is_occupied(x, y));

   Queen q(++maximum_figure_ID, x, y);
   
#ifndef NDEBUG
   std::cout << "adding queen " << q.getID() << " at\t(" << x << ", " << y << ").\n";
#endif
   
   // insert into STL container and obtain iterator to it
   this->figures.insert(q);
   auto q_iter = this->figures.find(q);

   // x value
   if(this->f_x.count(x) == 0) this->f_x[x] = FigureSet();
   this->f_x[x].insert(*q_iter);
   
   // y value
   if(this->f_y.count(y) == 0) this->f_y[y] = FigureSet();
   this->f_y[y].insert(*q_iter);
   
   // x+y value
   if(this->f_x_plus_y.count(x+y) == 0) this->f_x_plus_y[x+y] = FigureSet();
   this->f_x_plus_y[x+y].insert(*q_iter);
   
   // x-y value
   if(this->f_x_minus_y.count(x-y) == 0) this->f_x_minus_y[x-y] = FigureSet();
   this->f_x_minus_y[x-y].insert(*q_iter);
}

long nqueens::Configuration::count_threats() const
{
   long num_threats = 0;
   
   for(auto i = this->f_x.begin(); i != this->f_x.end(); i++)
      if(i->second.size() >= 2)
      {
#ifndef NDEBUG
         std::cout << "\n\t\t" << i->second.size() << "\twith x = " << i->first << ".  "
                   << "\t(Contribution: " << 2 * (i->second.size() - 1) << ").";
#endif
         num_threats += 2 * (i->second.size() - 1);
      }
#ifndef NDEBUG
   std::cout << "\n";
#endif
   
   for(auto i = this->f_y.begin(); i != this->f_y.end(); i++)
      if(i->second.size() >= 2)
      {
#ifndef NDEBUG
         std::cout << "\n\t\t" << i->second.size() << "\twith y = " << i->first << ".  "
                   << "\t(Contribution: " << 2 * (i->second.size() - 1) << ").";
#endif
         num_threats += 2 * (i->second.size() - 1);
      }
#ifndef NDEBUG
   std::cout << "\n";
#endif
   
   for(auto i = this->f_x_plus_y.begin(); i != this->f_x_plus_y.end(); i++)
      if(i->second.size() >= 2)
      {
#ifndef NDEBUG
         std::cout << "\n\t\t" << i->second.size() << "\twith x+y = " << i->first << "."
                   << "\t(Contribution: " << 2 * (i->second.size() - 1) << ").";
#endif
         num_threats += 2 * (i->second.size() - 1);
      }
#ifndef NDEBUG
   std::cout << "\n";
#endif
   
   for(auto i = this->f_x_minus_y.begin(); i != this->f_x_minus_y.end(); i++)
      if(i->second.size() >= 2)
      {
#ifndef NDEBUG
         std::cout << "\n\t\t" << i->second.size() << "\twith x-y = " << i->first << "."
                   << "\t(Contribution: " << 2 * (i->second.size() - 1) << ").";
#endif
         num_threats += 2 * (i->second.size() - 1);
      }
#ifndef NDEBUG
   std::cout << "\n";
#endif

   return num_threats;
}

bool nqueens::Configuration::is_occupied(int x, int y) const
{
   if(this->f_x.count(x) == 0) return false;
   
   auto fsetx = this->f_x.find(x);
   for(auto i = fsetx->second.begin(); i != fsetx->second.end(); i++)
   {
      assert(i->getx() == x);
      if(i->gety() == y) return true;
   }
   return false;
}
