#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <map>  // std::map
#include <set>  // std::set

#include "figure.h"

namespace nqueens
{
   struct Board 
   {
      int lx = 0;  // size in x direction
      int ly = 0;  // size in y direction
   };

   using FigureSet = std::set<Figure>;
   using Fields = std::map<int, FigureSet>;

   class Configuration 
   {
    public:
      Configuration() = delete;
      Configuration(Board in_board): board{in_board} {}
      
      // forbid copying
      Configuration(const Configuration&) = delete; 
      Configuration& operator=(const Configuration&) = delete; 
      
      Board get_board() const { return this->board; }
      
      void add_queen_at(int x, int y);
      
      bool is_occupied(int x, int y) const;

      long count_threats() const;

    private:
      Board board;  // contain the board as a property
      std::set<Figure> figures{};  // contain the figures through STL data structure
      
      Fields f_x{};  // figures with given x 
      Fields f_y{};  // figures with given y
      Fields f_x_plus_y{};  // figures with given x+y
      Fields f_x_minus_y{};  // figures with given x-y
   };
}

#endif
