#include "configuration.h"
#include "random-config.h"

#include <cassert>  // assert
#include <chrono>  // time data to initialize random number generator
#include <cstdlib>  // std::rand, std::srand, RAND_MAX

using namespace nqueens;

void nqueens::add_random_queens(Configuration* c, int N_queens)
{
   assert(c->get_board().lx * c->get_board().ly >= N_queens);
   
   // random seed to initialize the RNG
   std::srand(
      std::chrono::duration_cast<std::chrono::microseconds>(
         std::chrono::high_resolution_clock::now().time_since_epoch()
      ).count()
   );

   // factor applied to RNG in x direction
   double rscale_x = c->get_board().lx / ((double)RAND_MAX + 1.0);  
   // factor applied to RNG in y direction
   double rscale_y = c->get_board().ly / ((double)RAND_MAX + 1.0);  
   
   for(int i = 0; i < N_queens; i++)
   {
      int x, y;
      do {
         x = (int)(rscale_x*std::rand());
         y = (int)(rscale_y*std::rand());
      } while(c->is_occupied(x, y));
      c->add_queen_at(x, y);
   }
}
